<?php
/* 
   Name: bloo_session.classes.php
   Author: Bill Roberts
   Created: 4/1/2005
   Purpose: Contains main session object for Bloo
   $Revision: 1.14 $
   
Copyright (C) 2005-2008  Bill Roberts (bill@outofthebloo.com)

This program is free software; you can redistribute it and/or
modify it under the terms of the GNU General Public License
as published by the Free Software Foundation; either version 2
of the License, or (at your option) any later version.

GPL URL: http://www.gnu.org/licenses/gpl.html

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program; if not, write to the Free Software
Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
*/

/* Bloo-specific security roles */
define( "ROLE_POSTER", 3 );

/* Other bloo specific definitions */
define( "TABLE_BASED", "table" );
define( "REGISTRATION_MAX_SAVE_LENGTH", 64000 );
define( "EVENT_SESSION_INIT", "eventSessionInit" );

/**************************************************************/
/* Define the BlooSession class                               */
/* BlooSession - the main session object for a Bloo system    */
/**************************************************************/
class BlooSession extends PhooSession {
   var $name = "BlooSession";
   var $coreInventory = array( );
   var $themeInventory = array( );
   var $extensionInventory = array( );
   var $tableBasedOverlay = array( );
   var $timeZone;
   
   function BlooSession( ) {
      parent::PhooSession( );

      /* place the bootstrap setting in the core inventory for bloo_session */
      $this->coreInventory[ "bloo_session" ] = array( "loadable"=> true
                                                     ,"directory" => $row[ "snap_on_directory" ]
                                                    );
   }

   /* initialize the session */
   function initialize( $aServer, $aDatabase, $aUserId, $aPassword, $aSettingsTable ) {
      /* initialize for reading settings */
      $this->setVar( "settings_table", $aSettingsTable, "APPLICATION" );

      /* Connect to the database */
      $this->connect( $aServer, $aDatabase, $aUserId, $aPassword );

      if ( $this->isError( ) ) {
         return;
      }

      /* load settings */
      /* Create a snapOnQuery to load the settings */
      $s =& $this->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = ROLE_EVERYONE;

      /* version check: verify that the version on the DB is the same version in the code */
      $s->command = "=|Version.get|=";
      $s->process( );
      while ( $row = $s->getResultRow( ) ) {
         $dbVersion = $row[ "bloo_version" ];
      }

      /* if no version found, just skip this check */
      if ( $dbVersion > "" ) {
         if ( $dbVersion != BLOO_VERSION ) {
            $this->setError( "The version of Bloo represented by the database (version " . $dbVersion . 
                             ") is different than the version of Bloo in the code release (version " . BLOO_VERSION . 
                             "). Please have the blog administrator run the upgrade program.", MSG_FATAL );
            return;
         }
      }      
      
      $s->command = "=|Settings.list.get|=";
      $s->process( );
      /* for each resulting row, set the application variable */
      while ( $row = $s->getResultRow( ) ) {
         /* trap for a final slash in the site_url variable - if it's there, remove */
         if ( $row[ "setting_name" ] == "site_url" ) {
            $value = $row[ "setting_value" ];
            if ( substr( $value, strlen( $value )-1, 1 ) == "/" ) { /* trim the slash */
               $row[ "setting_value" ] = substr( $value, 0, strlen( $value ) - 1 ); 
            }
         }
         $this->setSetting( $row[ "setting_name" ], $row[ "setting_value" ] );
      }

      /* load event handlers */
      $this->loadEvents( "=|Event.list.active.get|=" );

      /* load the inventory of snapOns */
      $this->loadSnapOnInventory( );
      
      /* load Core SnapOn Package Descriptors */
      $snapOnPath = $this->getSetting( "core_path" );
      $this->loadDescriptorsFromPath( $snapOnPath, "core" ); 
      
      /* now that snapons are loaded, log the session in - if user previously logged in */

      /* if user login id information passed, attempt a login */
      $loginId = $this->getVar( "user_login_id", "POST" );
      $password = md5( $this->getVar( "user_password", "POST" ) );
      if ( isset( $loginId ) && isset( $password ) ) {
         $this->logIn( $loginId, $password );
         unset( $loginId );
         unset( $password );
         if ( $this->isError( ) ) {
            /* Since a bad login is not a fatal error, and really doesn't affect the rest of the 
               blog (just will be set to the not logged in state), clean the error message
               and set a session message */
            $error = $this->getError( );
            $this->setSessionMessage( $error, MSG_FATAL );
         }
      }
      else { /* see if a logout is called for */
         $action = $this->getVar( FORM_ACTION, "GET" );
         if ( $action == "logout" ) {
            $this->logOut( );
         }
         else {
            /* see if user previously logged in */
            $this->checkPreviousLogin( );
         }      
      }

      /* set the allowed post types to be viewed, based on whether the current user is logged in or not */
      /* 1 = public, 3 = private */
      if ( $this->isLoggedIn( ) ) {
         /* also, set an application variable with the user id */
         $user = $this->getUserInformation( );
         $this->setVar( "user_id", $user[ "user_id" ], "APPLICATION" );
         
         $this->setSetting( "allowed_post_type_list", "1,3" );
      }
      else {
         $this->setSetting( "allowed_post_type_list", "1" );
      }
         
      $extensionPath = $this->getSetting( "extension_path" );
      $this->loadDescriptorsFromPath( $extensionPath, "extension" ); 
      
      /* load the current bloo version in the overlays, and as a setting */
      $this->addOverlay( "Bloo.version", "template", BLOO_VERSION, "Current Bloo version", 
                         "bloo_session.classes.php", "core" );
      $this->setSetting( "bloo_version", BLOO_VERSION );

      /* set the current time zone */
      $timeZoneId = $this->getSetting( "time_zone_id" );

      if ( !isset( $timeZoneId ) ) {
         $timeZoneId = TZ_GMT;  /* just set to GMT */
      }
      $this->timeZone = new PhooTimeZone( $timeZoneId );

      /* fire any initialization events */ 
      $this->fireEvents( EVENT_SESSION_INIT );
      
      /* determine the current theme */
      $this->determineCurrentTheme( );
      
      $themePath = $this->getSetting( "theme_path" );
      $this->inventoryThemes( $themePath );
      
      /* path should be revised for the subdirectories, so reload */
      $themePath = $this->getSetting( "theme_path" );

      $this->setupThemes( );      
      
      $this->loadDescriptorsFromPath( $themePath, "theme" ); 
      
      /* add table-based descriptors to the descriptor arrays */
      $this->addTableBasedThemeDescriptors( );

      /* cache table-based overlays */
      $this->cacheTableBasedOverlays( );
      
      /* load the current theme snapOn (which will also load any themes it cascades from */
      $currentTheme = $this->getSetting( "current_theme" );      
      $this->loadSnapOn( $currentTheme );

   }

   /* this will load an inventory of all snapons of all types (core, extension, theme) in the system */
   function loadSnapOnInventory( ) {
      $s =& $this->snapOnFactory( "PhooQuery", $this );
      $s->securityRole = ROLE_EVERYONE;
      $s->command = "=|SnapOns.get|=";
      $s->process( );
      
      /* store each resulting row in the inventory */
      while ( $row = $s->getResultRow( ) ) {
         switch( $row[ "snap_on_type" ] ) {
            case "core" :
               $this->coreInventory[ $row[ "snap_on_name" ] ] = array( "loadable"=> true
                                                                      ,"directory" => $row[ "snap_on_directory" ]
                                                                     );

               break;
               
            case "extension" :
               if ( $row[ "snap_on_activated_flag" ] == "Y" ) {
                  $loadable = true;
               }
               else {
                  $loadable = false;
               }

               $this->extensionInventory[ $row[ "snap_on_name" ] ] = array( "loadable" => $loadable
                                                                           ,"source" => $row[ "snap_on_source" ]
                                                                           ,"directory" => $row[ "snap_on_directory" ]
                                                                           ,"descriptor" => $row[ "snap_on_descriptor" ]
                                                                          );
               break;
               
            case "theme" :
               $descriptor = $row[ "snap_on_descriptor" ];
               if ( isset( $descriptor ) && $descriptor > "" ) {
                  $x = new XML2ArrayParser( $descriptor, true );
                  if ( $x->isError( ) ) {
                     return;
                  }      

                  $cascadesFrom = $x->tagData[ "SNAPON" ][0][ "CASCADES_FROM_SNAP_ON" ];
                  $displayName = $x->tagData[ "SNAPON" ][0][ "DISPLAY_NAME" ];
                  $author = $x->tagData[ "SNAPON" ][0][ "AUTHOR" ];
                  $createDate = $x->tagData[ "SNAPON" ][0][ "CREATE_DATE" ];
               }                  

               $this->themeInventory[ $row[ "snap_on_name" ] ] = 
                  array( "cascades_from_snap_on" => $cascadesFrom
                        ,"theme_name" => $displayName
                        ,"display_name" => $displayName /* note: not sure why the redundancy */
                        ,"theme_author" => $author
                        ,"create_date" => $createDate
                        ,"directory" => TABLE_BASED
                        ,"loadable" => false );
               break;
         }
      }
   }
   
   /* determines the current theme */
   function determineCurrentTheme( ) {
      $themeOverridden = false;
      
      /* load the default theme */
      $currentTheme = $this->getSetting( "default_theme" );
      /* if the user is logged on, check for a preview theme request */
      if ( $this->isLoggedIn( ) ) {
         $themePreview = $this->getVar( "theme_preview", "GET" );
         if ( isset( $themePreview ) ) {
            $currentTheme = $themePreview;
            /* store the preview theme in a cookie. Preview will last no more than one hour */
            $this->setVar( "preview_theme", $themePreview, "COOKIE", time()+60*60 );
            $themeOverridden = true;
         }
         else {
            /* check for command to clear the cookie */
            $themePreviewStop = $this->getVar( "theme_preview_stop", "GET" );
            if ( $themePreviewStop == 1 ) {
               /* clear the cookie */
               $this->clearVar( "preview_theme", "COOKIE" );
            }
            else {
               /* check for a preview theme in the cookie */
               $previewTheme = $this->getVar( "preview_theme", "COOKIE" );
               if ( isset( $previewTheme ) ) {
                  $currentTheme = $previewTheme;
                  $themeOverridden = true;
               }
            }
         }
      }
      else {
         /* sefault user id variable to 0 - no one is logged on */
         $this->setVar( "user_id", 0, "APPLICATION" );
      }

      if ( !$themeOverridden ) {
         /* check for a theme_selector theme */
         $themeSelectorTheme = $this->getSetting( "theme_selector_theme" );
         if ( $themeSelectorTheme != $currentTheme ) {
            $currentTheme = $themeSelectorTheme;
         }
      }      

      $this->setVar( "current_theme", $currentTheme, "APPLICATION" );
   }
   
   /* determines the themes that cascade into the current theme */
   function inventoryThemes( $aThemePath ) {
      $themeDir = explode( ";", $aThemePath );
      
      $revisedThemePath = ""; 
      foreach( $themeDir as $dir ) {
         /* load any subdirectories too */
         if ( $dh = opendir( $dir ) ) {
            while ( ( $subDir = readdir( $dh ) ) !== false ) {
               if ( $subDir != "." && $subDir != ".." && is_dir( $dir . "/" . $subDir ) ) {
                  /* recursively inventory themes found in subdirectories */
                  $this->InventoryThemes( $dir . "/" . $subDir );
                  /* add recursive directories to the theme path */
                  $themePath = $this->getSetting( "theme_path" );
                  $this->setSetting( "theme_path", $themePath . ";" . $dir . "/" . $subDir );
               }
            }
         }
         $themeDescriptorList = $this->getEregFileNames( $dir, ".+\.descriptor\.xml$" );

         if ( is_array( $themeDescriptorList ) ) {
            foreach( $themeDescriptorList as $descriptor ) {
               /* verify it's a theme, load the array */
               $x = new XML2ArrayParser( $descriptor );
               if ( $x->isError( ) ) {
                  return;
               }      

               if ( $x->tagData[ "SNAPON" ][0][ "TYPE" ] == "theme" ) {
                  $name = $x->tagData[ "SNAPON" ][0][ "NAME" ];
                  /* note: if already in the table, don't load - table-based themes take precedence */
                  if ( !isset( $this->themeInventory[ $name ] ) ) {
                     $this->themeInventory[ $name ] = 
                        array( "cascades_from_snap_on" => $x->tagData[ "SNAPON" ][0][ "CASCADES_FROM_SNAP_ON" ]
                              ,"display_name" => $x->tagData[ "SNAPON" ][0][ "DISPLAY_NAME" ]
                              ,"directory" => $dir
                              ,"loadable" => false );
                  }
               }
            }
         }
      }
   } 
      
   function setupThemes( ) {
      /* inventory built, start with current theme and work backwards, marking all the themes in the
         cascade path as loadable, and building the stylesheet list */
      $currentTheme = $this->getSetting( "current_theme" );

      /* verify that the current theme is a real one */
      if ( !isset( $this->themeInventory[ $currentTheme ] ) ) {
         /* not real! Try the default theme */
         $currentTheme = $this->getSetting( "default_theme" );
         if ( !isset( $this->themeInventory[ $currentTheme ] ) ) {
            /* ok, this is very, very bad. Set it to Bloo Base */
            $currentTheme = "bloo_base";

            if ( !isset( $this->themeInventory[ $currentTheme ] ) ) {
               /* catastrophe! . . . Throw an error */
               $this->setError( "Illegal theme specified: " . $this->getSetting( "current_theme" ) . ". Could not set " .
                                "to another theme. Aborting . . .", MSG_FATAL );
               return;
            }
         }

         /* if we made it this far, we're good. Set the current theme variable */
         $this->setSetting( "current_theme", $currentTheme );
      }

      /* start with the current theme and work backwards */
      $theme = $currentTheme;
      while ( $theme > "" ) {
         $this->themeInventory[ $theme ][ "loadable" ] = true;
         $theme = $this->themeInventory[ $theme ][ "cascades_from_snap_on" ];
      }
   }

   function addTableBasedThemeDescriptors( ) {
      foreach ( $this->themeInventory as $themeTag => $descriptor ) {
         if ( $descriptor[ "directory" ] == TABLE_BASED ) {
            
            $this->snapOn[ $themeTag ] = array( "type" => "theme"
                                               ,"display_name" => $descriptor[ "theme_name" ]
                                               ,"author" => $descriptor[ "theme_author" ]
                                               ,"create_date" => $descriptor[ "create_date" ]
                                               ,"cascades_from_snap_on" => $descriptor[ "cascades_from_snap_on" ]
                                               ,"loaded" => false
                                               ,"loadable" => $descriptor[ "loadable" ]
                                               ,"source" => TABLE_BASED
                                              );            
         }
      }
   } 
   
   /* load table-based overlays */
   function cacheTableBasedOverlays( ) {
      /* will load overlays in the order that they cascade in */
      $theme = $this->getSetting( "current_theme" );
      $themeList = array( );
      
      while ( $theme > "" ) {
         array_unshift( $themeList, $theme );
         
         $theme = $this->themeInventory[ $theme ][ "cascades_from_snap_on" ];
      }
      
      /* place "all" as the first theme in the list */
      array_unshift( $themeList, "all" );

      foreach ( $themeList as $theme ) {
         $this->setVar( "loop_theme", $theme, "PRIVATE" );      
      
         $s =& $this->snapOnFactory( "PhooQuery", $this );
         $s->securityRole = ROLE_EVERYONE;
         $s->command = "=|Overlays.get|=";
         $s->process( );
         
         /* for each resulting row, set the application variable */
         while ( $row = $s->getResultRow( ) ) {
            $row[ "overlay_source" ] = TABLE_BASED;
            $this->tableBasedOverlay[ $row[ "overlay_name" ] ] = $row;

            /* if this is a unique overlay - meaning, a created overlay with a new name that doesn't
               match an existing overlay in one of Bloo's packages, load it now (otherwise it will
               never load) */
            if ( !isset( $this->packageOverlay[ $row[ "overlay_name" ] ] ) ) {
               $syntax = $this->unClean( $row[ "overlay_syntax" ] );
         
               $this->addOverlay( $row[ "overlay_name" ]
                                 ,$row[ "overlay_type" ]
                                 ,$syntax
                                 ,$row[ "overlay_description" ]
                                 ,$row[ "overlay_source" ] 
                                 ,TABLE_BASED
                                 ,$row[ "overlay_pend_to" ]
                                 ,$row[ "overlay_pend_type" ]
                                 ,$row[ "overlay_theme" ]
                                 ,$row[ "overlay_id" ] );
            }
            
         }
      }
   }
   
   /* extends ancestor - contains special processing for themes and extensions */
   function isLoadable( &$aDescriptorTagData ) {
      $type = $aDescriptorTagData[ "SNAPON" ][0][ "TYPE" ];

      $loadable = true;
      
      switch( $type ) {
         case "core" : 
            $name = $aDescriptorTagData[ "SNAPON" ][0][ "NAME" ];
            $loadable = $this->coreInventory[ $name ][ "loadable" ];
            break;

         case "theme" :
            $name = $aDescriptorTagData[ "SNAPON" ][0][ "NAME" ];
            $loadable = $this->themeInventory[ $name ][ "loadable" ];
            break;

         case "extension" :
            $name = $aDescriptorTagData[ "SNAPON" ][0][ "NAME" ];
            $loadable = $this->extensionInventory[ $name ][ "loadable" ];
            break;
      }
      
      return $loadable;
   }

   /* extends ancestor - checks for style sheet or inline css and adds to the appropriate
      application variable */
   function setSnapOnLoaded( $aSnapOnName ) {
      $loadedSet = parent::setSnapOnLoaded( $aSnapOnName );
      if ( $loadedSet ) {
         /* see if there's a style sheet associated with this SnapOn. If there is,
            get it in the style sheet list */
         if ( isset( $this->snapOn[ $aSnapOnName ][ "style_sheet" ] ) &&
              $this->snapOn[ $aSnapOnName ][ "style_sheet" ] > "" ) {
            $styleSheet = $this->getSetting( "style_sheet" );
            if ( $styleSheet > "" ) {
               $styleSheet .= ";"; /* add the delimiter */
            }

            $directory = $this->snapOn[ $aSnapOnName ][ "directory" ];
            $this->setSetting( "style_sheet", $styleSheet . $directory . "/" . 
                               $this->snapOn[ $aSnapOnName ][ "style_sheet" ] );
         }

         /* see if there is inline CSS associated with this SnapOn. If there is,
            add it to the inline CSS */
         if ( isset( $this->snapOn[ $aSnapOnName ][ "inline_css" ] ) &&
              $this->snapOn[ $aSnapOnName ][ "inline_css" ] > "" ) {
            $inlineCSS = $this->getSetting( "inline_css" );
            if ( $inlineCSS > "" ) {
               $inlineCSS .= "\n"; /* add the delimiter */
            }

            $this->setSetting( "inline_css", $inlineCSS .  
                               $this->snapOn[ $aSnapOnName ][ "inline_css" ] );
         }
            
      }
   }

   /* extends ancestor - forces hierarchy on overlay adds, so that a theme overlay can't be
      overlayed by a core overlay, and an extension overlay can't be overlayed by a theme overlay
    */
   function addOverlay( $aName, $aType, $aSyntax, $aDescription, $aSource, $aSnapOnType, $aPendTo = null
                       ,$aPendType = null, $aTheme = "all", $aId = 0 ) {

      /* see if overlay already exists */
      if ( isset( $this->overlay[ $aName ] ) ) {
         
         $storedSnapOnType = $this->overlay[ $aName ][ "snap_on_type" ];

         if ( $storedSnapOnType == "extension" ) {
            if ( $aSnapOnType == "core" || $aSnapOnType == "theme" ) {
               return; /* don't overlay */
            }
         }
         elseif ( $storedSnapOnType == "theme" ) {
            if ( $aSnapOnType == "core" ) {
               return; /* don't overlay */
            }
         }
      }

      /* still here - call the parent */
      parent::addOverlay( $aName, $aType, $aSyntax, $aDescription, $aSource, $aSnapOnType, $aPendTo
                         ,$aPendType, $aTheme, $aId );
   }

   /* extends ancestor - loads table-based overlay if requested */
   function getOverlay( $aName, $aWithAppendages = true ) {
      if ( isset( $this->tableBasedOverlay[ $aName ] ) ) {
         $ovl = $this->tableBasedOverlay[ $aName ];

         $syntax = $this->unClean( $ovl[ "overlay_syntax" ] );
         
         $this->addOverlay( $ovl[ "overlay_name" ], $ovl[ "overlay_type" ], $syntax
                           ,$ovl[ "overlay_description" ], $ovl[ "overlay_source" ] 
                           ,TABLE_BASED
                           ,$ovl[ "overlay_pend_to" ], $ovl[ "overlay_pend_type" ]
                           ,$ovl[ "overlay_theme" ], $ovl[ "overlay_id" ] );
      }

      return parent::getOverlay( $aName, $aWithAppendages );
   }

   function loadOverlayPackage( $aOverlayName ) {
      /* check to see if the overlay is a table-based overlay. If so, there is no package to load */
      if ( isset( $this->tableBasedOverlay[ $aOverlayName ] ) ) {
         return;
      }

      parent::loadOverlayPackage( $aOverlayName );
   }
   
   /* returns the value of a setting */
   function getSetting( $aSettingName ) {
      return $this->getVar( $aSettingName, "APPLICATION" );
   }

   function setSetting( $aSettingName, $aValue ) {
      $this->setVar( $aSettingName, $aValue, "APPLICATION" );
   }

   /* returns an array - the first element is a formatted local date version of the GMT date string passed. 
      The date string should be a GMT date, and is assumed to be so. If no date string is passed,
      return the current local time. The second array element is the time zone abbreviation, and the third
      is a logical specifying whether this is DST */
   function getLocalDate( $aDateFormat, $aGMTDate=null ) {
      $timeInZone = $this->timeZone->getTimeInZone( $aDateFormat, $aGMTDate );       
      
      return $timeInZone;
   }

   /* session function that will place a message in the system log */
   function logMessage( $aPackage, $aMessage ) {
      $l =& $this->snapOnFactory( "SystemLogWriter", $this );
      $l->setMessage( $aPackage, $aMessage );
      $l->process( );
   }

   /* get and set functions for session messages */
   function setSessionMessage( $aSessionMessage, $aSessionMessageType ) {
      $this->setVar( "session_message", $aSessionMessage, "SESSION" );
      $this->setVar( "session_message_type", $aSessionMessageType, "SESSION" );
   }
   
   function getSessionMessage( ) {
      $sessionMessage = $this->getVar( "session_message", "SESSION" );
      $sessionMessageType = $this->getVar( "session_message_type", "SESSION" );

      return array( "session_message" => $sessionMessage 
                   ,"session_message_type" => $sessionMessageType );
   }

   function clearSessionMessage( ) {
      $this->clearVar( "session_message", "SESSION" );
      $this->clearVar( "session_message_type", "SESSION" );
   }
}

class BlooTimeZoneDropDown extends TimeZoneDropDown {
   var $name = "BlooTimeZoneDropDown";
   var $securityRole = ROLE_ADMINISTRATOR;
   var $templateHeader = "=|TimeZone.dropdown.header|=";
   var $template = "=|TimeZone.dropdown.item|=";
   var $templateFooter = "=|TimeZone.dropdown.footer|=";
}
?>
